﻿-- Addon:	ChiBar: functional part
-- Author:	Rakhan(EU-Azshara)

if select(2, UnitClass("player")) ~= "MONK" then
    return
end


------------------------------------------
--		Functions						--
------------------------------------------

-- Function for hiding/showing the Addon
function ChiBar:enable()
	if ChiBarDB.enable then
		ChiBar.mainFrame:Hide()
		ChiBarDB.enable = false
		print(string.format(ChiBarLocalization.HIDE))
		ChiBar:unregisterEvents()
	else
		ChiBar.showMainFrame()
		ChiBarDB.enable = true
		print(string.format(ChiBarLocalization.SHOW))
		ChiBar:registerEvents()
	end
end

-- Function for changing the texture
function ChiBar:changeTexture()
	for index = 1, 5 do
		ChiBar.Textures0[index]:SetTexture(ChiBar.textures[((ChiBarDB.savedTexture*2)-1)])
		ChiBar.Textures1[index]:SetTexture(ChiBar.textures[(ChiBarDB.savedTexture*2)])
	end
end

-- Function for showing mainFrame under conditions of autohide
function ChiBar:showMainFrame()
	if not (ChiBarDB.autohide and UnitAffectingCombat("player") == nil) then
		ChiBar.mainFrame:Show()
	end
end

-- Function for setting the default parameters
function ChiBar:defaultParameters()
	ChiBarDB.x = 0
	ChiBarDB.y = 0
	ChiBarDB.from = "CENTER"
	ChiBarDB.to = "CENTER"
	ChiBarDB.scale = 1
	ChiBarDB.alpha = 1
	ChiBarDB.enable = true
	ChiBarDB.autohide = false
	ChiBarDB.savedTexture = 1
	ChiBarDB.grow1 = true
	ChiBarDB.grow2 = true
end

-- Function for resetting the ChiBar
function ChiBar:reset()
	ChiBar:defaultParameters()
	ChiBar:registerEvents()
	ChiBar:changeTexture()
	ChiBar:settingUpOrientation()
	ChiBar.mainFrame:SetPoint(ChiBarDB.from, UIParent, ChiBarDB.to, ChiBarDB.x, ChiBarDB.y)
	ChiBar.mainFrame:SetScale(ChiBarDB.scale)
	ChiBar.mainFrame:SetAlpha(ChiBarDB.alpha)
	ChiBar.mainFrame:Show()
	ChiBar.Widgets.enableBox:SetChecked(ChiBarDB.enable)
	ChiBar.Widgets.autohideBox:SetChecked(ChiBarDB.autohide)
	ChiBar.Widgets.scaleSlider:SetValue(ChiBarDB.scale)
	ChiBar.Widgets.opacitySlider:SetValue(ChiBarDB.alpha)
	ChiBar.Widgets.verticalBox:SetChecked(ChiBarDB.grow1)
	ChiBar.Widgets.horizontalBox:SetChecked(not ChiBarDB.grow1)
	ChiBar.Widgets.leftbottomBox:SetChecked(ChiBarDB.grow1)
	ChiBar.Widgets.righttopBox:SetChecked(not ChiBarDB.grow1)

	UIDropDownMenu_SetSelectedID(ChiBar.Widgets.dropdownMenu, ChiBarDB.savedTexture)
	print(string.format(ChiBarLocalization.RESET))
end

-- Function for scaling the ChiBar
function ChiBar:scaleUI(scale)
	ChiBarDB.scale = floor(scale*100)/100
	ChiBar.mainFrame:SetScale(ChiBarDB.scale)
	ChiBar.Widgets.currScaleText:SetText(ChiBarDB.scale)
end

-- Function for adjust the opacity of ChiBar
function ChiBar:adjustOpacity(alpha)
	ChiBarDB.alpha = floor(alpha*100)/100
	ChiBar.mainFrame:SetAlpha(ChiBarDB.alpha)
	ChiBar.Widgets.currOpacityText:SetText(ChiBarDB.alpha)
end

-- Function for update the GUI on Talentchange (Ascension)
function ChiBar:updateStructure()
	if UnitPowerMax("player", 12) == 4 then
		ChiBar.Bubbles[5]:Hide()
	else
		ChiBar.Bubbles[5]:Show()
	end
	ChiBar:settingUpOrientation()
end

-- Function for updating the horizontal/vertical orientation
function ChiBar:updateVertHor()
	if ChiBarDB.grow1 then
		ChiBarDB.grow1 = false
		ChiBar.Widgets.leftbottomText:SetText(ChiBarLocalization.OP_GROW_BOTTOM)
		ChiBar.Widgets.righttopText:SetText(ChiBarLocalization.OP_GROW_TOP)
		ChiBar.Widgets.verticalBox:SetChecked(ChiBarDB.grow1)
		ChiBar.Widgets.horizontalBox:SetChecked(not ChiBarDB.grow1)
	else
		ChiBarDB.grow1 = true
		ChiBar.Widgets.leftbottomText:SetText(ChiBarLocalization.OP_GROW_LEFT)
		ChiBar.Widgets.righttopText:SetText(ChiBarLocalization.OP_GROW_RIGHT)
		ChiBar.Widgets.verticalBox:SetChecked(ChiBarDB.grow1)
		ChiBar.Widgets.horizontalBox:SetChecked(not ChiBarDB.grow1)
	end
	ChiBar:settingUpOrientation()
end

-- Function for updating the grow direction
function ChiBar:updateGrowthDirec()
	if ChiBarDB.grow2 then
		ChiBarDB.grow2 = false
		ChiBar.Widgets.leftbottomBox:SetChecked(ChiBarDB.grow2)
		ChiBar.Widgets.righttopBox:SetChecked(not ChiBarDB.grow2)
	else
		ChiBarDB.grow2 = true
		ChiBar.Widgets.leftbottomBox:SetChecked(ChiBarDB.grow2)
		ChiBar.Widgets.righttopBox:SetChecked(not ChiBarDB.grow2)
	end
	ChiBar:settingUpOrientation()
end

-- Function for update the GUI(amount of available Chi)
function ChiBar:updateChi()
	local i = 0
	local currChi = UnitPower("player", 12)
	local maxChi = UnitPowerMax("player", 12)
	for i = 1, currChi do
		ChiBar.Textures1[i]:Show()
	end
	for i = (UnitPower("player", 12)+1), maxChi do
		ChiBar.Textures1[i]:Hide()
	end
end

-- Function for toggle Autohide out of Combat
function ChiBar:autohide()
	if ChiBarDB.autohide then
		ChiBarDB.autohide = false
		ChiBar.mainFrame:Show()
		print(string.format(ChiBarLocalization.AUTOHIDE_OFF))
	else
		ChiBarDB.autohide = true
		if UnitAffectingCombat("player") == nil then
			ChiBar.mainFrame:Hide()
		end
		print(string.format(ChiBarLocalization.AUTOHIDE_ON))
	end
end

-- Function for enable Lock
function ChiBar:lockFrame()
	ChiBar.mainFrame:SetMovable(false)
	ChiBar.mainFrame:EnableMouse(false)
	ChiBarDB.from, _, ChiBarDB.to, ChiBarDB.x, ChiBarDB.y = ChiBar.mainFrame:GetPoint()
	if ChiBarDB.autohide and UnitAffectingCombat("player") == nil then
		ChiBar.mainFrame:Hide()
	end
	ChiBar.BGTexture:Hide()
	ChiBar.BGTexture2:Hide()
	print(string.format(ChiBarLocalization.LOCK))
end

-- Function for disable Lock
function ChiBar:unlockFrame()
	InterfaceOptionsFrame_Show() 
	ChiBar.mainFrame:SetMovable(true)
	ChiBar.mainFrame:EnableMouse(true)
	ChiBar.mainFrame:Show()
	StaticPopup_Show("CHIBAR_UNLOCKFRAME")
	ChiBar.BGTexture:Show()
	ChiBar.BGTexture2:Show()
	print(string.format(ChiBarLocalization.UNLOCK))
end



------------------------------------------
--		EventHandling					--
------------------------------------------

function ChiBar_OnEvent(self, event, arg1, arg2, ...)
	-- Player entered Vehicle
	if event == "UNIT_ENTERED_VEHICLE" and arg1 == "player" then
		ChiBar.mainFrame:Hide()
		
	-- Player exit Vehicle
	elseif event == "UNIT_EXITED_VEHICLE" and arg1 == "player" then
		ChiBar:showMainFrame()
		
	-- Player enter Combat while Autohide is on
	elseif event == "PLAYER_REGEN_DISABLED" and ChiBarDB.autohide then
		ChiBar.mainFrame:Show()
		
	-- Player exit Combat while Autohide is on
	elseif event == "PLAYER_REGEN_ENABLED" and ChiBarDB.autohide then
		ChiBar.mainFrame:Hide()
		
	-- Player changed Talents
	elseif event == "UNIT_MAXPOWER" then
		ChiBar:updateStructure()
		ChiBar:updateChi()
		
	-- Player's Chi changed
	elseif event == "UNIT_POWER" and arg1 == "player" and arg2 == "LIGHT_FORCE" then
		ChiBar:updateChi()
		
	-- Player enters PetBattle
	elseif event == "PET_BATTLE_OPENING_START" then
		ChiBar.mainFrame:Hide()
		
	-- Player exits PetBattle
	elseif event == "PET_BATTLE_CLOSE" then
		ChiBar:showMainFrame()
		
	-- Player logging in - only used because UnitPowerMax() returns 0 after ADDON_LOADED
	elseif event == "PLAYER_LOGIN" then
		ChiBar.mainFrame:UnregisterEvent("PLAYER_LOGIN")
		ChiBar:updateStructure()
		ChiBar:updateChi()
		
	-- initial loading of the addon'S settings after loading the scriptfiles
	elseif event == "ADDON_LOADED" and arg1 == "ChiBar" then
		if ChiBarDB == nil then
			print("Creating ChiBarDB")
			ChiBarDB = {}
			ChiBar:defaultParameters()
		end
		if ChiBarDB.alpha == nil then
			ChiBarDB.alpha = 1
			ChiBarDB.grow1 = true
			ChiBarDB.grow2 = true
		end
		ChiBar:createUI()
		ChiBar:createOptionsUI()
		print(string.format(ChiBar.VERSIONTEXT))
		print(string.format(ChiBarLocalization.GREETINGS))
		ChiBar.mainFrame:UnregisterEvent("ADDON_LOADED")
		if ChiBarDB.autohide and UnitAffectingCombat("player") == nil then
			ChiBar.mainFrame:Hide()
		end
		if not ChiBarDB.enable then
			ChiBar.mainFrame:Hide()
			ChiBar.unregisterEvents()
		end
	end
end

-- Function for registering Events
function ChiBar:unregisterEvents()
	ChiBar.mainFrame:UnregisterEvent("UNIT_POWER")
	ChiBar.mainFrame:UnregisterEvent("UNIT_ENTERED_VEHICLE")
	ChiBar.mainFrame:UnregisterEvent("UNIT_EXITED_VEHICLE")
	ChiBar.mainFrame:UnregisterEvent("UNIT_MAXPOWER")
	ChiBar.mainFrame:UnregisterEvent("PLAYER_REGEN_DISABLED")
	ChiBar.mainFrame:UnregisterEvent("PLAYER_REGEN_ENABLED")
	ChiBar.mainFrame:UnregisterEvent("PET_BATTLE_OPENING_START")
	ChiBar.mainFrame:UnregisterEvent("PET_BATTLE_CLOSE")
end

-- Function for unregistering Events
function ChiBar:registerEvents()
	ChiBar.mainFrame:RegisterEvent("UNIT_POWER")
	ChiBar.mainFrame:RegisterEvent("UNIT_ENTERED_VEHICLE")
	ChiBar.mainFrame:RegisterEvent("UNIT_EXITED_VEHICLE")
	ChiBar.mainFrame:RegisterEvent("UNIT_MAXPOWER")
	ChiBar.mainFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	ChiBar.mainFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
	ChiBar.mainFrame:RegisterEvent("PET_BATTLE_OPENING_START")
	ChiBar.mainFrame:RegisterEvent("PET_BATTLE_CLOSE")
end

-- Registring Events the UI should react to
ChiBar.mainFrame:RegisterEvent("ADDON_LOADED")
ChiBar.mainFrame:RegisterEvent("PLAYER_LOGIN")
ChiBar:registerEvents()
ChiBar.mainFrame:SetScript("OnEvent", ChiBar_OnEvent)
ChiBar.mainFrame:SetScript("OnDragStart", ChiBar.mainFrame.StartMoving)
ChiBar.mainFrame:SetScript("OnDragStop", ChiBar.mainFrame.StopMovingOrSizing)


------------------------------------------
--		SlashCommand					--
------------------------------------------

SLASH_CHIBAR1 = "/cb"
SLASH_CHIBAR2 = "/chibar"
SlashCmdList["CHIBAR"] = function(msg)
	InterfaceOptionsFrame_OpenToCategory(ChiBar.opanel)
end


------------------------------------------
--		End of Script					--
------------------------------------------